@@ -17,10 +17,12 @@ import dash
 import plotly.graph_objects as go
 from dash import Dash
 from jupyter_dash import JupyterDash
+from plotly.basedatatypes import BaseFigure
 from trace_updater import TraceUpdater
 
-from .figure_resampler_interface import AbstractFigureAggregator
 from ..aggregation import AbstractSeriesAggregator, EfficientLTTB
+from .figure_resampler_interface import AbstractFigureAggregator
+from .utils import is_figure, is_fr
 
 
 class FigureResampler(AbstractFigureAggregator, go.Figure):
@@ -28,7 +30,7 @@ class FigureResampler(AbstractFigureAggregator, go.Figure):
 
     def __init__(
         self,
-        figure: go.Figure = None,
+        figure: BaseFigure | dict = None,
         convert_existing_traces: bool = True,
         default_n_shown_samples: int = 1000,
         default_downsampler: AbstractSeriesAggregator = EfficientLTTB(),
@@ -39,12 +41,27 @@ class FigureResampler(AbstractFigureAggregator, go.Figure):
         show_mean_aggregation_size: bool = True,
         verbose: bool = False,
     ):
-        if figure is None:
-            figure = go.Figure()
+        # Parse the figure input before calling `super`
+        if is_figure(figure) and not is_fr(figure):  # go.Figure
+            # Base case, the figure does not need to be adjusted
+            f = figure
+        else:
+            # Create a new figure object and make sure that the trace uid will not get
+            # adjusted when they are added.
+            f = self._get_figure_class(go.Figure)()
+            f._data_validator.set_uid = False
+
+            if isinstance(figure, BaseFigure):  # go.FigureWidget or AbstractFigureAggregator
+                # A base figure object, we first copy the layout and grid ref
+                f.layout = figure.layout
+                f._grid_ref = figure._grid_ref
+                f.add_traces(figure.data)
+            elif isinstance(figure, (dict, list)):
+                # A single trace dict or a list of traces
+                f.add_traces(figure)
 
-        assert isinstance(figure, go.Figure)
         super().__init__(
-            figure,
+            f,
             convert_existing_traces,
             default_n_shown_samples,
             default_downsampler,
@@ -53,6 +70,23 @@ class FigureResampler(AbstractFigureAggregator, go.Figure):
             verbose,
         )
 
+        if isinstance(figure, AbstractFigureAggregator):
+            # Copy the `_hf_data` if the previous figure was an AbstractFigureAggregator
+            # and adjust the default `max_n_samples` and `downsampler`
+            self._hf_data.update(
+                self._copy_hf_data(figure._hf_data, adjust_default_values=True)
+            )
+
+            # Note: This hack ensures that the this figure object initially uses
+            # data of the whole view. More concretely; we create a dict
+            # serialization figure and adjust the hf-traces to the whole view
+            # with the check-update method (by passing no range / filter args)
+            with self.batch_update():
+                graph_dict: dict = self._get_current_graph()
+                update_indices = self._check_update_figure_dict(graph_dict)
+                for idx in update_indices:
+                    self.data[idx].update(graph_dict["data"][idx])
+
         # The FigureResampler needs a dash app
         self._app: JupyterDash | Dash | None = None
         self._port: int | None = None
